#include <linux/module.h>
#include <linux/init.h>
#include <linux/gpio.h>
#include <linux/hrtimer.h>
#include <linux/ktime.h>

 
MODULE_DESCRIPTION("EpicClock Module");
MODULE_AUTHOR("SI");
MODULE_LICENSE("GPL");
 
static struct hrtimer hr_timer;
static ktime_t ktime_period;
 
int enstatus = 0;
 
#define BLINK_DELAY   4
#define MS_TO_NS(x)(x * 1E6L)   


int digits[10][8] = 
{
{1,1,1,1,1,1,0},//0
{0,1,1,0,0,0,0},//1
{1,1,0,1,1,0,1},//2
{1,1,1,1,0,0,1},//3
{0,1,1,0,0,1,1},//4
{1,0,1,1,0,1,1},//5
{1,0,1,1,1,1,1},//6
{1,1,1,0,0,0,0},//7
{1,1,1,1,1,1,1},//8
{1,1,1,1,0,1,1},//9
};
// TODO de completat cu maparea curentă 
int seg[7] = {24,22,18,23,17,25,10};
int en[6] = {8,4,9,15,7,11};

/* Selects the segments of the digits
 * /param x The number to display
 */
void digit(int x)
{
	// TODO1 Display digits correctly
    //
}

/* Enables the driver to power one of the digits
 * \param enbit Line of the driver to be enabled, -1 for none.
 * \param point Activates separation points
 */
void enable(int enbit)
{
	int i;

	for (i = 0; i < 6; i++)
		if (i == enbit)
			gpio_set_value(en[i],1);
		else 
			gpio_set_value(en[i],0);
}
 
/* Timer Callback to change current digit */ 
enum hrtimer_restart my_timer_func(struct hrtimer * timer)
{
    ktime_t kt_now;
    int ret_overrun;
	int cifra = 0;
 
	/* current digit shown */
	enstatus = (enstatus + 1 ) % 6;


	// TODO2: Obtain current digit to display
	
	digit(cifra);
	enable(enstatus);

    kt_now = hrtimer_cb_get_time(&hr_timer);
    ret_overrun = hrtimer_forward(&hr_timer, kt_now, ktime_period);

    return HRTIMER_RESTART;
}
 
static int __init leds_init(void)
{
	int i;
    ktime_t ktime;
	printk(KERN_INFO "epicclock: loading\n");

	/* Initialize All signals to output, but off */
	for (i = 0; i < 7; i++)
		gpio_direction_output(seg[i],digits[1][i]);
	for (i = 0; i < 6; i++)
		gpio_direction_output(en[i],1);
 
	/*
	 * Set up the timer for the first time
	 */
    ktime_period = ktime_set( 0, MS_TO_NS(BLINK_DELAY) );
    hrtimer_init(&hr_timer, CLOCK_MONOTONIC, HRTIMER_MODE_REL);
    hr_timer.function = &my_timer_func;
    hrtimer_start( &hr_timer, ktime, HRTIMER_MODE_REL );
	return 0;
}
 
static void __exit leds_cleanup(void)
{
	printk(KERN_INFO "epicclock: unloading...\n");

	/* disable timer */
    int ret;

    ret = hrtimer_cancel( &hr_timer );
    if (ret)
        printk(KERN_ALERT "Could not disable timer!\n");
	/* disable all digits and points */
	enable(-1);
}
 
module_init(leds_init);
module_exit(leds_cleanup);
